"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validate = exports.plugin = void 0;
const path_1 = require("path");
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
const pascalCase = (str) => str.length > 1
    ? str.charAt(0).toUpperCase() + str.slice(1)
    : str.toUpperCase();
const getTypeScriptTypeName = (namedTypeNode, scalars) => {
    if (scalars && scalars.includes(namedTypeNode.name.value)) {
        return `schemaTypes.Scalars['${namedTypeNode.name.value}']`;
    }
    return `schemaTypes.${namedTypeNode.name.value}`;
};
const generateTypeScriptType = (typeNode, scalars) => {
    if (typeNode.kind === "NonNullType") {
        return typeNode.type.kind === "NamedType"
            ? getTypeScriptTypeName(typeNode.type, scalars)
            : `${generateTypeScriptType(typeNode.type.type, scalars)}[]`;
    }
    const typeName = typeNode.kind === "NamedType"
        ? getTypeScriptTypeName(typeNode, scalars)
        : `${generateTypeScriptType(typeNode.type, scalars)}[]`;
    return `schemaTypes.Maybe<${typeName}>`;
};
const generateTypeScriptVariableType = (args, scalars, avoidInputValueOptional) => [
    "{",
    ...args.map((arg) => {
        return `${arg.name.value}${arg.type.kind === "NonNullType" || avoidInputValueOptional ? "" : "?"}: ${generateTypeScriptType(arg.type, scalars)},`;
    }),
    "}",
].join("\n");
const queryTemplate = (operation, fieldName, argumentType, returnType) => `export type ${operation}${pascalCase(fieldName)}Handler = Handler<AppSyncResolverEvent<${argumentType}>, ${returnType}>`;
const DEFAULT_AVOID_OPTIONALS = {
    object: false,
    inputValue: false,
};
const normalizeAvoidOptionals = (avoidOptionals) => {
    if (typeof avoidOptionals === "boolean") {
        return {
            object: avoidOptionals,
            inputValue: avoidOptionals,
        };
    }
    return {
        ...DEFAULT_AVOID_OPTIONALS,
        ...avoidOptionals,
    };
};
const createImportPath = (outputFile, schemaFile) => {
    const splitFilePath = (filePath) => {
        const paths = filePath.split(path_1.sep);
        const filename = paths.pop() || "";
        return {
            path: paths.join(path_1.sep),
            basename: path_1.basename(filename, path_1.extname(filename)),
            filename,
        };
    };
    const output = splitFilePath(outputFile);
    const schema = splitFilePath(schemaFile);
    const relativePath = path_1.relative(output.path, schema.path) || ".";
    return relativePath + path_1.sep + schema.basename;
};
exports.plugin = (schema, _, config, info) => {
    const scalars = Object.keys(visitor_plugin_common_1.buildScalars(schema, config.scalars || {}));
    const avoidInputValueOptional = normalizeAvoidOptionals(config.avoidOptionals)
        .inputValue;
    // Query
    const queryType = schema.getQueryType();
    const queries = queryType && queryType.astNode && queryType.astNode.fields
        ? queryType.astNode.fields.map((field) => queryTemplate("Query", field.name.value, field.arguments
            ? generateTypeScriptVariableType(field.arguments, scalars, avoidInputValueOptional)
            : "null", generateTypeScriptType(field.type, scalars)))
        : [];
    // Mutation
    const mutationType = schema.getMutationType();
    const mutations = mutationType && mutationType.astNode && mutationType.astNode.fields
        ? mutationType.astNode.fields.map((field) => queryTemplate("Mutation", field.name.value, field.arguments
            ? generateTypeScriptVariableType(field.arguments, scalars, avoidInputValueOptional)
            : "null", generateTypeScriptType(field.type, scalars)))
        : [];
    // Subscriptions
    const subscriptionType = schema.getSubscriptionType();
    const subscriptions = subscriptionType &&
        subscriptionType.astNode &&
        subscriptionType.astNode.fields
        ? subscriptionType.astNode.fields.map((field) => queryTemplate("Subscription", field.name.value, field.arguments
            ? generateTypeScriptVariableType(field.arguments, scalars, avoidInputValueOptional)
            : "null", generateTypeScriptType(field.type, scalars)))
        : [];
    return {
        prepend: [
            `import { Handler } from 'aws-lambda';
import * as schemaTypes from "${createImportPath((info === null || info === void 0 ? void 0 : info.outputFile) || "", config.schemaPath)}"
export type AppSyncResolverEvent<ARGUMENTS = null> = {
  arguments: ARGUMENTS
  source: null
  result: null
  identity: {
    sub: string
    issuer: string
    username: string
    claims: { [key: string]: string }
    sourceIP: string
    defaultAuthStrategy: string
  }
  request: {
    headers: { [key: string]: string }
  }
  info: {
    parentTypeName: string
    fieldName: string
    variables: any
  }
  error: null
  prev: null
  stash: {}
  outErrors: []
}`,
        ],
        content: [...queries, ...mutations, ...subscriptions].join("\n"),
    };
};
exports.validate = (schema, documents, config, outputFile, allPlugins) => {
    if (!config.schemaPath) {
        throw new Error(`You must specify "schemaPath" in graphql-codegen-aws-lambda-types plugin configuration!`);
    }
};
